@@ -13,4 +13,10 @@ usb_find_common_endpoints(struct usb_host_interface *alt,
 		struct usb_endpoint_descriptor **int_out);
 #endif /* < 4.12 */
 
+#if LINUX_VERSION_IS_LESS(5,15,0)
+#define usb_check_bulk_endpoints LINUX_BACKPORT(usb_check_bulk_endpoints)
+bool usb_check_bulk_endpoints(
+		const struct usb_interface *intf, const u8 *ep_addrs);
+#endif /* < 5.15 */
+
 #endif /* __BACKPORT_LINUX_USB_H */
